//
//  LGOMModel.m
//  LifeGameOnMac
//
//  Created by narita on 2016/08/31.
//  Copyright © 2016年 narita. All rights reserved.
//

#import "LGOMModel.h"

@interface LGOMModel ()

// 上下に1列、左右に1列をそれぞれ追加することで、次の状態を計算する時の境界条件分岐を省略する。

@property NSMutableArray* cells;

@end

@implementation LGOMModel

#pragma mark -
#pragma mark class methods

+ (nullable instancetype) modelWithSize:(LGOMSize)inSize
{
    return [[LGOMModel alloc] initWithSize:inSize];
}

#pragma mark -
#pragma mark init methods

- (nullable instancetype) initWithSize:(LGOMSize)inSize
{
    NSAssert((inSize.height >= 1) && (inSize.width >= 1), nil);
    
    self = [super init];
    
    if( self != nil )
    {
        srand((unsigned int) time(NULL));
        
        self.cellsSize = inSize;
        
        self.cells = [[NSMutableArray alloc] initWithCapacity:
                      (self.cellsSize.width + 2) * ( self.cellsSize.height + 2)];
        [self randomizeAll];
    }
    
    return self;
}

#pragma mark -
#pragma mark dealloc

#pragma mark -
#pragma mark NSCopying, hash, isEqual:

#pragma mark -
#pragma mark NSCoding methods

#pragma mark -
#pragma mark awakeFromNib-like methods

#pragma mark -
#pragma mark action methods

- (void) clearAll
{
    NSAssert( self.cells != nil, nil);
    
    [self.cells removeAllObjects];

    NSInteger theLength = (self.cellsSize.width + 2) * ( self.cellsSize.height + 2);

    for(int i = 0; i < theLength; i++)
    {
        [self.cells addObject:[NSNumber numberWithBool:NO]];
    }
}

- (void) randomizeAll
{
    NSAssert( self.cells != nil, nil);
    
    [self clearAll];
    
    for(NSInteger y = 0; y < self.cellsSize.height; y++)
    {
        for(NSInteger x = 0; x < self.cellsSize.height; x++)
        {
            LGOMPoint thePoint = {x, y};
            
            [self setAlive:(random() % 2)
                   atPoint:thePoint];
        }
    }
}

- (BOOL) aliveAtPoint:(LGOMPoint)inPoint
{
    NSAssert( self.cells != nil, nil);
    NSAssert( inPoint.x <  self.cellsSize.width + 1, nil);
    NSAssert( inPoint.x >= -1, nil);
    NSAssert( inPoint.y <  self.cellsSize.height + 1, nil);
    NSAssert( inPoint.y >= -1, nil);
    
    // 境界分だけindexを増やす
    inPoint.x += 1;
    inPoint.y += 1;
    
    NSNumber* theValue = [self.cells objectAtIndex:
                          inPoint.y * self.cellsSize.width + inPoint.x];
    
    return [theValue boolValue];
}

- (void) setAlive:(BOOL)inAlive atPoint:(LGOMPoint)inPoint
{
    NSAssert( self.cells != nil, nil);
    NSAssert( inPoint.x <  self.cellsSize.width + 1, nil);
    NSAssert( inPoint.x >= -1, nil);
    NSAssert( inPoint.y <  self.cellsSize.height + 1, nil);
    NSAssert( inPoint.y >= -1, nil);
    
    NSNumber* theValue = [NSNumber numberWithBool:inAlive];
    
    // 境界分だけindexを増やす
    inPoint.x += 1;
    inPoint.y += 1;

    [self.cells replaceObjectAtIndex:inPoint.y * self.cellsSize.width + inPoint.x
                          withObject:theValue];
}

- (void) next
{
    NSMutableArray* theNextCells = [self.cells mutableCopy];

    // ループさせる
    for(NSInteger y = 0; y < self.cellsSize.height; y++)
    {
        for(NSInteger x = 0; x < self.cellsSize.height; x++)
        {
            LGOMPoint thePoint = {x, y};
            
            BOOL      theCurrentValue = [self aliveAtPoint:thePoint];
            NSInteger theNeighValue = [self sumAliveNeighborCellsAtPoint:thePoint];
            BOOL      theNextValue = theCurrentValue;
            
            if( theCurrentValue )
            {
                if( !((theNeighValue == 2) || (theNeighValue == 3)) )
                {
                    theNextValue = NO;
                }
            }
            else
            {
                if( theNeighValue == 3 )
                {
                    theNextValue = YES;
                }
            }
            
            // 結果の書き込み
            {
                NSNumber* theValue = [NSNumber numberWithBool:theNextValue];
                
                // 境界分だけindexを増やす
                thePoint.x += 1;
                thePoint.y += 1;
                
                [theNextCells replaceObjectAtIndex:thePoint.y * self.cellsSize.width + thePoint.x
                                        withObject:theValue];
                
            }
        }
    }
    
    self.cells = theNextCells;
}

- (NSInteger) sumAliveNeighborCellsAtPoint:(LGOMPoint)inPoint
{
    NSInteger theResult = 0;
    LGOMPoint thePoint = inPoint;
    
    thePoint = inPoint;
    thePoint.x -= 1;thePoint.y -= 1;
    if( [self aliveAtPoint:thePoint] ) { theResult++; }
    
    thePoint = inPoint;
                    thePoint.y -= 1;
    if( [self aliveAtPoint:thePoint] ) { theResult++; }

    thePoint = inPoint;
    thePoint.x += 1;thePoint.y -= 1;
    if( [self aliveAtPoint:thePoint] ) { theResult++; }
    

    thePoint = inPoint;
    thePoint.x -= 1;
    if( [self aliveAtPoint:thePoint] ) { theResult++; }
    
    thePoint = inPoint;
    thePoint.x += 1;
    if( [self aliveAtPoint:thePoint] ) { theResult++; }
    

    thePoint = inPoint;
    thePoint.x -= 1;thePoint.y += 1;
    if( [self aliveAtPoint:thePoint] ) { theResult++; }
    
    thePoint = inPoint;
                    thePoint.y += 1;
    if( [self aliveAtPoint:thePoint] ) { theResult++; }

    thePoint = inPoint;
    thePoint.x += 1;thePoint.y += 1;
    if( [self aliveAtPoint:thePoint] ) { theResult++; }
    
    return theResult;
}


#pragma mark -
#pragma mark delegate/datasource methods

#pragma mark -
#pragma mark accessor methods (in pairs)

#pragma mark -
#pragma mark Utility methods

- (NSString*) description
{
    NSAssert( self.cells != nil, nil);

    NSMutableString* theResult = [NSMutableString string];

    for(NSInteger y = 0; y < self.cellsSize.height; y++)
    {
        for(NSInteger x = 0; x < self.cellsSize.height; x++)
        {
            LGOMPoint thePoint = {x, y};
            
            BOOL      theCurrentValue = [self aliveAtPoint:thePoint];

            if(theCurrentValue)
            {
                [theResult appendString:@"O"];
            }
            else
            {
                [theResult appendString:@"."];
            }
        }
        
        [theResult appendString:@"\r"];
    }
    
    return [theResult copy];
}

- (void)   gameBoardSetup:(NSString*)inNewLifeGameBoard
{
    NSAssert( self.cells != nil, nil);
    
    NSRange theRange = {0, 1};
    
    for(NSInteger y = 0; y < self.cellsSize.height; y++)
    {
        for(NSInteger x = 0; x < self.cellsSize.height; x++)
        {
            LGOMPoint thePoint = {x, y};
            
            NSString* theSubString = [inNewLifeGameBoard substringWithRange:theRange];
            
            if( [theSubString isEqualToString:@"O"] )
            {
                [self setAlive:YES atPoint:thePoint];
            }
            else if( [theSubString isEqualToString:@"."] )
            {
                [self setAlive:NO atPoint:thePoint];
            }
            else
            {
                NSAssert( NO, nil);
            }
            
            theRange.location += 1;
        }
        
        NSString* theSubString = [inNewLifeGameBoard substringWithRange:theRange];
        
        if( [theSubString isEqualToString:@"\r"] )
        {
            // 改行
        }
        else
        {
            NSAssert( NO, nil);
        }
        
        theRange.location += 1;
    }
}

@end
