//
//  ViewController.m
//  LifeGameOnMac
//
//  Created by narita on 2016/08/31.
//  Copyright © 2016年 narita. All rights reserved.
//

#import "LGOMViewController.h"

#import "LGOMApplicationDelegate.h"
#import "LGOMModel.h"


@implementation LGOMViewController

#pragma mark -
#pragma mark class methods

#pragma mark -
#pragma mark init methods

#pragma mark -
#pragma mark dealloc

#pragma mark -
#pragma mark NSCopying, hash, isEqual:

#pragma mark -
#pragma mark NSCoding methods

#pragma mark -
#pragma mark awakeFromNib-like methods

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    NSButtonCell* theCell = self.matrix.prototype;
    [theCell setButtonType:NSOnOffButton];
    
    
    LGOMModel* theModel = [LGOMAppDelegate model];
    NSAssert( theModel != nil, nil);

    self.representedObject = theModel;
    
    [self updateUI];
}

#pragma mark -
#pragma mark NSViewController life cycle methods

#pragma mark -
#pragma mark action methods

- (IBAction) start:(id)sender
{
    if( self.timer == nil )
    {
        self.matrix.enabled = NO;
        [self repeat:nil];
    }
}

- (IBAction) stop:(id)sender
{
    [self.timer invalidate];
    self.timer = nil;
    
    self.matrix.enabled = YES;
}

- (IBAction) random:(id)sender
{
    NSAssert( self.representedObject != nil, nil);

    [self stop:nil];
    [self.representedObject randomizeAll];
    [self updateUI];
}


- (IBAction) clear:(id)sender
{
    NSAssert( self.representedObject != nil, nil);

    [self stop:nil];
    [self.representedObject clearAll];
    [self updateUI];
}

- (IBAction) repeat:(id)sender
{
    NSAssert( self.representedObject != nil, nil);

    [self.timer invalidate];
    self.timer = nil;

    [self next:nil];
    
    self.timer = [NSTimer scheduledTimerWithTimeInterval:1.0
                                                  target:self
                                                selector:@selector(repeat:)
                                                userInfo:nil
                                                 repeats:NO];
}

- (IBAction) next:(id)sender
{
    NSAssert( self.representedObject != nil, nil);

    LGOMModel* theModel = self.representedObject;
    
    [theModel next];
    [self updateUI];
}

- (IBAction) toggleCell:(id)sender
{
    NSAssert( self.matrix == sender, nil);
    
    LGOMModel* theModel = self.representedObject;
    
    NSAssert( self.representedObject != nil, nil);
    NSAssert( self.matrix.numberOfColumns == theModel.cellsSize.width,  nil);
    NSAssert( self.matrix.numberOfRows    == theModel.cellsSize.height, nil);
    
    
    for(NSInteger y = 0; y < theModel.cellsSize.height; y++)
    {
        for(NSInteger x = 0; x < theModel.cellsSize.height; x++)
        {
            LGOMPoint thePoint = {x, y};
            
            NSCell* theCell = [self.matrix cellAtRow:y
                                              column:x];
            [theModel setAlive:theCell.state
                       atPoint:thePoint];

        }
    }
}

#pragma mark -
#pragma mark delegate/datasource methods

#pragma mark -
#pragma mark accessor methods (in pairs)

- (void)setRepresentedObject:(id)representedObject
{
    [super setRepresentedObject:representedObject];
    
    if( representedObject != nil )
    {
        LGOMModel* theModel = representedObject;
        
        for(; self.matrix.numberOfColumns != theModel.cellsSize.width; )
        {
            [self.matrix addColumn];
        }
        
        for(; self.matrix.numberOfRows != theModel.cellsSize.height; )
        {
            [self.matrix addRow];
        }
    }
}

- (LGOMModel*) representedObject
{
    return [super representedObject];
}

#pragma mark -
#pragma mark Utility methods

- (void) updateUI
{
    LGOMModel* theModel = self.representedObject;
    
    NSAssert( self.representedObject != nil, nil);
    NSAssert( self.matrix.numberOfColumns == theModel.cellsSize.width,  nil);
    NSAssert( self.matrix.numberOfRows    == theModel.cellsSize.height, nil);
    
    
    for(NSInteger y = 0; y < theModel.cellsSize.height; y++)
    {
        for(NSInteger x = 0; x < theModel.cellsSize.height; x++)
        {
            LGOMPoint thePoint = {x, y};
            
            BOOL      theCurrentValue = [theModel aliveAtPoint:thePoint];
            
            [self.matrix setState:theCurrentValue
                            atRow:y
                           column:x];
        }
    }
}

@end
